/**
 * @file test-utils.h
 * @author Peter Rockai (mornfall) <mornfall@danill.sk>
 * @brief Utility functions for the unit tests
 */
#ifndef XGRIBARCH_TEST_UTILS_H
#define XGRIBARCH_TEST_UTILS_H

#include <wibble/tests.h>

#include <iostream>

namespace arki {
namespace tests {

}
}

#endif
